"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = nextPageConfig;
var _core = require("@babel/core");
var _constants = require("next/constants");
const CONFIG_KEY = 'config';
function replaceBundle(path, t) {
  path.parentPath.replaceWith(t.program([t.variableDeclaration('const', [t.variableDeclarator(t.identifier(_constants.STRING_LITERAL_DROP_BUNDLE), t.stringLiteral(`${_constants.STRING_LITERAL_DROP_BUNDLE} ${Date.now()}`))])], []));
}
function errorMessage(state, details) {
  const pageName = (state.filename || '').split(state.cwd || '').pop() || 'unknown';
  return `Invalid page config export found. ${details} in file ${pageName}. See: https://nextjs.org/docs/messages/invalid-page-config`;
}
function nextPageConfig({
  types: t
}) {
  return {
    visitor: {
      Program: {
        enter(path, state) {
          path.traverse({
            ExportDeclaration(exportPath, exportState) {
              if (_core.types.isExportNamedDeclaration(exportPath) && exportPath.node.specifiers?.some(specifier => {
                return (t.isIdentifier(specifier.exported) ? specifier.exported.name : specifier.exported.value) === CONFIG_KEY;
              }) && _core.types.isStringLiteral(exportPath.node.source)) {
                throw new Error(errorMessage(exportState, 'Expected object but got export from'));
              }
            },
            ExportNamedDeclaration(exportPath, exportState) {
              if (exportState.bundleDropped || !exportPath.node.declaration && exportPath.node.specifiers.length === 0) {
                return;
              }
              const config = {};
              const declarations = [...(exportPath.node.declaration?.declarations || []), exportPath.scope.getBinding(CONFIG_KEY)?.path.node].filter(Boolean);
              for (const specifier of exportPath.node.specifiers) {
                if ((t.isIdentifier(specifier.exported) ? specifier.exported.name : specifier.exported.value) === CONFIG_KEY) {
                  if (_core.types.isStringLiteral(exportPath.node.source)) {
                    throw new Error(errorMessage(exportState, `Expected object but got import`));
                  } else if (_core.types.isIdentifier(specifier.local)) {
                    if (_core.types.isImportSpecifier(exportPath.scope.getBinding(specifier.local.name)?.path.node)) {
                      throw new Error(errorMessage(exportState, `Expected object but got import`));
                    }
                  }
                }
              }
              for (const declaration of declarations) {
                if (!_core.types.isIdentifier(declaration.id, {
                  name: CONFIG_KEY
                })) {
                  continue;
                }
                let {
                  init
                } = declaration;
                if (_core.types.isTSAsExpression(init)) {
                  init = init.expression;
                }
                if (!_core.types.isObjectExpression(init)) {
                  const got = init ? init.type : 'undefined';
                  throw new Error(errorMessage(exportState, `Expected object but got ${got}`));
                }
                for (const property of init.properties) {
                  if (_core.types.isSpreadElement(property)) {
                    throw new Error(errorMessage(exportState, `Property spread is not allowed`));
                  }
                  const {
                    name
                  } = property.key;
                  if (_core.types.isIdentifier(property.key, {
                    name: 'amp'
                  })) {
                    if (!_core.types.isObjectProperty(property)) {
                      throw new Error(errorMessage(exportState, `Invalid property "${name}"`));
                    }
                    if (!_core.types.isBooleanLiteral(property.value) && !_core.types.isStringLiteral(property.value)) {
                      throw new Error(errorMessage(exportState, `Invalid value for "${name}"`));
                    }
                    config.amp = property.value.value;
                  }
                }
              }
              if (config.amp === true) {
                if (!exportState.file?.opts?.caller.isDev) {
                  replaceBundle(exportPath, t);
                }
                exportState.bundleDropped = true;
                return;
              }
            }
          }, state);
        }
      }
    }
  };
}