"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = optimizeHookDestructuring;
const isHook = /^use[A-Z]/;
const isBuiltInHook = /^use(Callback|Context|DebugValue|Effect|ImperativeHandle|LayoutEffect|Memo|Reducer|Ref|State)$/;
function optimizeHookDestructuring({
  types: t
}) {
  const visitor = {
    CallExpression(path, state) {
      const {
        onlyBuiltIns,
        lib
      } = state.opts;
      const libs = lib && (lib === true ? ['react', 'preact/hooks'] : [lib].flat());
      if (!t.isVariableDeclarator(path.parent)) return;
      if (!t.isArrayPattern(path.parent.id)) return;
      const hookName = path.node.callee.name;
      if (libs) {
        const binding = path.scope.getBinding(hookName);
        if (!binding || binding.kind !== 'module') return;
        const specifier = binding.path.parent.source.value;
        if (!libs.includes(specifier)) return;
      }
      if (!(onlyBuiltIns ? isBuiltInHook : isHook).test(hookName)) return;
      path.parent.id = t.objectPattern(path.parent.id.elements.reduce((patterns, element, index) => {
        if (element === null) {
          return patterns;
        }
        return patterns.concat(t.objectProperty(t.numericLiteral(index), element));
      }, []));
    }
  };
  return {
    name: 'optimize-hook-destructuring',
    visitor: {
      Program(path, state) {
        path.traverse(visitor, state);
      }
    }
  };
}