"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;
var _nodePath = require("node:path");
function _default({
  types: t
}) {
  return {
    visitor: {
      ImportDeclaration(path, state) {
        const source = path.node.source.value;
        if (source !== 'next/dynamic') return;
        const defaultSpecifier = path.get('specifiers').find(specifier => {
          return specifier.isImportDefaultSpecifier();
        });
        if (!defaultSpecifier) return;
        const bindingName = defaultSpecifier.node.local.name;
        const binding = path.scope.getBinding(bindingName);
        if (!binding) {
          return;
        }
        binding.referencePaths.forEach(referencePath => {
          let callExpression = referencePath.parentPath;
          if (callExpression.isMemberExpression() && callExpression.node.computed === false) {
            const property = callExpression.get('property');
            if (!Array.isArray(property) && property.isIdentifier({
              name: 'Map'
            })) {
              callExpression = callExpression.parentPath;
            }
          }
          if (!callExpression.isCallExpression()) return;
          const callExpression_ = callExpression;
          let args = callExpression_.get('arguments');
          if (args.length > 2) {
            throw callExpression_.buildCodeFrameError('next/dynamic only accepts 2 arguments');
          }
          if (!args[0]) {
            return;
          }
          let loader;
          let options;
          if (args[0].isObjectExpression()) {
            options = args[0];
          } else {
            if (!args[1]) {
              callExpression_.node.arguments.push(t.objectExpression([]));
            }
            args = callExpression_.get('arguments');
            loader = args[0];
            options = args[1];
          }
          if (!options.isObjectExpression()) return;
          const options_ = options;
          const properties = options_.get('properties');
          const propertiesMap = {};
          properties.forEach(property => {
            const key = property.get('key');
            propertiesMap[key.node.name] = property;
          });
          if (propertiesMap.loadableGenerated) {
            return;
          }
          if (propertiesMap.loader) {
            loader = propertiesMap.loader.get('value');
          }
          if (propertiesMap.modules) {
            loader = propertiesMap.modules.get('value');
          }
          if (!loader || Array.isArray(loader)) {
            return;
          }
          const dynamicImports = [];
          const dynamicKeys = [];
          if (propertiesMap.ssr) {
            const ssr = propertiesMap.ssr.get('value');
            const nodePath = Array.isArray(ssr) ? undefined : ssr;
            if (nodePath) {
              const nonSSR = nodePath.node.type === 'BooleanLiteral' && nodePath.node.value === false;
              if (nonSSR && loader && state.file.opts.caller?.isServer) {
                loader.replaceWith(t.arrowFunctionExpression([], t.nullLiteral(), true));
              }
            }
          }
          loader.traverse({
            Import(importPath) {
              const importArguments = importPath.parentPath.get('arguments');
              if (!Array.isArray(importArguments)) return;
              const {
                node
              } = importArguments[0];
              dynamicImports.push(node);
              dynamicKeys.push(t.binaryExpression('+', t.stringLiteral(`${state.file.opts.caller?.pagesDir ? (0, _nodePath.relative)(state.file.opts.caller.pagesDir, state.file.opts.filename) : state.file.opts.filename} -> `), node));
            }
          });
          if (!dynamicImports.length) return;
          options.node.properties.push(t.objectProperty(t.identifier('loadableGenerated'), t.objectExpression(state.file.opts.caller?.isDev || state.file.opts.caller?.isServer ? [t.objectProperty(t.identifier('modules'), t.arrayExpression(dynamicKeys))] : [t.objectProperty(t.identifier('webpack'), t.arrowFunctionExpression([], t.arrayExpression(dynamicImports.map(dynamicImport => {
            return t.callExpression(t.memberExpression(t.identifier('require'), t.identifier('resolveWeak')), [dynamicImport]);
          }))))])));
          if (loader.isCallExpression()) {
            const arrowFunction = t.arrowFunctionExpression([], loader.node);
            loader.replaceWith(arrowFunction);
          }
        });
      }
    }
  };
}