"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _nodePath = require("node:path");
const isLoadIntentTest = process.env.NODE_ENV === 'test';
const isLoadIntentDevelopment = process.env.NODE_ENV === 'development';
function styledJsxOptions(options) {
  options = options || {};
  options.styleModule = 'styled-jsx/style';
  if (!Array.isArray(options.plugins)) {
    return options;
  }
  options.plugins = options.plugins.map(plugin => {
    if (Array.isArray(plugin)) {
      const [name, pluginOptions] = plugin;
      return [require.resolve(name), pluginOptions];
    }
    return require.resolve(plugin);
  });
  return options;
}
function supportsStaticESM(caller) {
  return !!caller?.supportsStaticESM;
}
var _default = (api, options = {}) => {
  const supportsESM = api.caller(supportsStaticESM);
  const isServer = api.caller(caller => !!caller && caller.isServer);
  const isCallerDevelopment = api.caller(caller => caller?.isDev);
  const isTest = isCallerDevelopment == null && isLoadIntentTest;
  const isDevelopment = isCallerDevelopment === true || isCallerDevelopment == null && isLoadIntentDevelopment;
  const isProduction = !(isTest || isDevelopment);
  const isBabelLoader = api.caller(caller => !!caller && (caller.name === 'babel-loader' || caller.name === 'next-babel-turbo-loader'));
  const useJsxRuntime = options['preset-react']?.runtime === 'automatic' || Boolean(api.caller(caller => !!caller && caller.hasJsxRuntime)) && options['preset-react']?.runtime !== 'classic';
  const presetEnvironmentConfig = {
    modules: 'auto',
    exclude: ['transform-typeof-symbol'],
    ...options['preset-env']
  };
  if ((isServer || isTest) && (!presetEnvironmentConfig.targets || !(typeof presetEnvironmentConfig.targets === 'object' && 'node' in presetEnvironmentConfig.targets))) {
    presetEnvironmentConfig.targets = {
      node: process.versions.node
    };
  }
  return {
    sourceType: 'unambiguous',
    presets: [[require('@babel/preset-env'), presetEnvironmentConfig], [require('@babel/preset-react'), {
      development: isDevelopment || isTest,
      ...(useJsxRuntime ? {
        runtime: 'automatic'
      } : {
        pragma: '__jsx'
      }),
      ...options['preset-react']
    }], [require('@babel/preset-typescript'), {
      allowNamespaces: true,
      ...options['preset-typescript']
    }]],
    plugins: [!useJsxRuntime && [require("./plugins/jsx-pragma"), {
      module: 'react',
      importAs: 'React',
      pragma: '__jsx',
      property: 'createElement'
    }], [require("./plugins/optimize-hook-destructuring"), {
      lib: true
    }], require('@babel/plugin-syntax-dynamic-import'), require('@babel/plugin-syntax-import-assertions'), require("./plugins/react-loadable-plugin"), [require('@babel/plugin-proposal-class-properties'), options['class-properties'] || {}], [require('@babel/plugin-proposal-object-rest-spread'), {
      useBuiltIns: true
    }], !isServer && [require('@babel/plugin-transform-runtime'), {
      corejs: false,
      helpers: true,
      regenerator: true,
      useESModules: supportsESM && presetEnvironmentConfig.modules !== 'commonjs',
      absoluteRuntime: isBabelLoader ? (0, _nodePath.dirname)(require.resolve('@babel/runtime/package.json')) : undefined,
      ...options['transform-runtime']
    }], [isTest && options['styled-jsx'] && options['styled-jsx']['babel-test'] ? require('styled-jsx/babel-test') : require('styled-jsx/babel'), styledJsxOptions(options['styled-jsx'])], require("./plugins/amp-attributes"), isProduction && [require('next/dist/compiled/babel/plugin-transform-react-remove-prop-types'), {
      removeImport: true
    }], isServer && require('@babel/plugin-syntax-bigint'), require('@babel/plugin-proposal-numeric-separator'), require('@babel/plugin-proposal-export-namespace-from')].filter(Boolean)
  };
};
exports.default = _default;